"""
AutoCoder Module - Part of Webscout
Provides automated code generation and manipulation capabilities.
"""

from .autocoder import *
from .autocoder_utiles import *

# __all__ = []  # Add your public module names here
